import {
  describe,
  expectTypeOf,
  globalExpect,
  it
} from "./chunk-FRFV7NXB.js";
import {
  World
} from "./chunk-QWOAABBK.js";
import "./chunk-GXV6P7V5.js";
import "./chunk-XFYQWM5B.js";

// src/world.test.ts
describe("World", () => {
  describe("get entities()", () => {
    it("returns an entity iterator", () => {
      const world = new World();
      world.createEntity({ color: "red" });
      world.createEntity({ color: "blue" });
      globalExpect(world.entities).toBeInstanceOf(Object);
      expectTypeOf(world.entities).toEqualTypeOf();
    });
  });
  describe("clearEntities()", () => {
    it("should clear all entities in the world and archetypes", () => {
      const world = new World();
      world.createEntity({ color: "red" });
      world.createEntity({ color: "blue" });
      world.archetype("color");
      globalExpect(world.entities.size).toBe(2);
      globalExpect(world.archetypes.size).toBe(1);
      world.clearEntities();
      globalExpect(world.entities.size).toBe(0);
      globalExpect(world.archetypes.size).toBe(1);
      for (const archetype of world.archetypes) {
        globalExpect(archetype.entities.size).toBe(0);
      }
    });
  });
  describe("createEntity()", () => {
    it("returns a new entity", () => {
      const world = new World();
      const entityA = world.createEntity({ color: "red" });
      globalExpect(entityA).toBeDefined();
      globalExpect(entityA.color).toBe("red");
    });
    it("should support initial empty entities as unique", () => {
      const world = new World();
      const entityA = world.createEntity();
      const entityB = world.createEntity();
      globalExpect(entityA).not.toBe(entityB);
      world.addEntityComponents(entityA, "color", "red");
      globalExpect(entityA.color).toBe("red");
      globalExpect(entityB.color).toBeUndefined();
      globalExpect(entityA).not.toBe(entityB);
    });
  });
  describe("deleteEntity()", () => {
    it("should delete an entity and return true", () => {
      const world = new World();
      const entity = world.createEntity({});
      globalExpect(world.deleteEntity(entity)).toBe(true);
    });
    it("should not delete an entity and return false", () => {
      const world = new World();
      globalExpect(world.deleteEntity({})).toBe(false);
    });
    it("should return false if the entity has already been deleted", () => {
      const world = new World();
      const entity = world.createEntity({});
      globalExpect(world.deleteEntity(entity)).toBe(true);
      globalExpect(world.deleteEntity(entity)).toBe(false);
    });
  });
  describe("addEntityComponents()", () => {
    it("should add components to entity", () => {
      const world = new World();
      const entity = world.createEntity();
      world.addEntityComponents(entity, "color", "red");
      world.addEntityComponents(entity, "rectangle", { width: 10, height: 10 });
      globalExpect(entity.color).toBe("red");
      globalExpect(entity.rectangle).toEqual({ width: 10, height: 10 });
    });
  });
  describe("removeEntityComponents()", () => {
    it("should remove entity components", () => {
      const world = new World();
      const entity = world.createEntity();
      world.addEntityComponents(entity, "color", "red");
      world.addEntityComponents(entity, "rectangle", {
        width: 10,
        height: 10
      });
      world.removeEntityComponents(entity, "color");
      globalExpect(entity.color).toBeUndefined();
      globalExpect(entity.rectangle).toEqual({ width: 10, height: 10 });
    });
    it("should not error if entity does not have components", () => {
      const world = new World();
      const entity = world.createEntity();
      world.addEntityComponents(entity, "color", "red");
      globalExpect(
        () => world.removeEntityComponents(entity, "rectangle")
      ).not.toThrow();
    });
    it("should not throw if entity does not exist and is not deleted", () => {
      const world = new World();
      globalExpect(
        () => world.removeEntityComponents({}, "color")
      ).not.toThrowError();
    });
    it.skip("should throw if entity has been mark for deletion");
  });
  describe("archetype()", () => {
    it("should return the correct archetype", () => {
      const world = new World();
      const entity = world.createEntity();
      const testTransform = { position: { x: 0, y: 0 } };
      world.addEntityComponents(entity, "transform", testTransform);
      globalExpect(world.archetype("transform").entities).toEqual(/* @__PURE__ */ new Set([entity]));
      globalExpect(world.archetype("color").entities).toEqual(/* @__PURE__ */ new Set());
    });
    it("should be updated correctly via removeEntityComponents()", () => {
      const world = new World();
      const entity = world.createEntity();
      const moving = world.archetype("transform", "velocity");
      globalExpect(moving.entities).toEqual(/* @__PURE__ */ new Set());
      world.addEntityComponents(entity, "transform", {
        position: { x: 0, y: 0 }
      });
      world.addEntityComponents(entity, "velocity", { x: 10, y: 10 });
      globalExpect(moving.entities).toEqual(/* @__PURE__ */ new Set([entity]));
      world.removeEntityComponents(entity, "transform");
      globalExpect(moving.entities).toEqual(/* @__PURE__ */ new Set());
    });
    it("should not return an archetype on partial component match", () => {
      const world = new World();
      world.createEntity({
        color: "red",
        transform: { position: { x: 0, y: 0 } },
        rectangle: { width: 10, height: 10 },
        velocity: { x: 10, y: 10 }
      });
      world.createEntity({
        color: "blue",
        transform: { position: { x: 1, y: 1 } }
      });
      const archetypeA = world.archetype(
        "color",
        "rectangle",
        "transform",
        "velocity"
      );
      const archetypeB = world.archetype("color", "transform");
      globalExpect(archetypeA).not.toBe(archetypeB);
      globalExpect(archetypeA.entities.size).toBe(1);
      globalExpect(archetypeB.entities.size).toBe(2);
    });
    it("should update correctly within systems", () => {
      const world = new World();
      const entity = world.createEntity({
        color: "red",
        transform: { position: { x: 0, y: 0 } }
      });
      world.createEntity({
        color: "blue",
        transform: { position: { x: 1, y: 1 } }
      });
      const renderSystemFactory = (world2) => {
        const renderables = world2.archetype("color", "transform");
        globalExpect(renderables.entities.size).toBe(2);
        let count = 0;
        return () => {
          if (count === 0) {
            const iterator = renderables.entities.values();
            {
              const entity2 = iterator.next().value;
              globalExpect(entity2.color).toBe("red");
              globalExpect(entity2.transform).toEqual({
                position: { x: 0, y: 0 }
              });
            }
            {
              const entity2 = iterator.next().value;
              globalExpect(entity2.color).toBe("blue");
              globalExpect(entity2.transform).toEqual({
                position: { x: 1, y: 1 }
              });
            }
            count++;
          } else {
            globalExpect(renderables.entities.size).toBe(1);
            const iterator = renderables.entities.values();
            const entity2 = iterator.next().value;
            globalExpect(entity2.color).toBe("blue");
            globalExpect(entity2.transform).toEqual({
              position: { x: 1, y: 1 }
            });
          }
        };
      };
      const renderSystem = renderSystemFactory(world);
      renderSystem();
      entity.color;
      world.removeEntityComponents(entity, "color");
      entity.color;
      renderSystem();
    });
  });
});
